<?php

namespace Drupal\highlightjs_input_filter\Plugin\Filter;

use Drupal\filter\FilterProcessResult;
use Drupal\filter\Plugin\FilterBase;

/**
 * Provides a filter to attach assets for <code> tags.
 *
 * @Filter(
 *   id = "filter_highlightjs",
 *   title = @Translation("Highlight code using highlight.js"),
 *   description = @Translation("Highlights code inside <code>&lt;pre&gt;&lt;code class='language-*'&gt;</code> tags."),
 *   type = Drupal\filter\Plugin\FilterInterface::TYPE_TRANSFORM_REVERSIBLE
 * )
 */
class HighlightJs extends FilterBase {

  /**
   * {@inheritdoc}
   */
  public function process($text, $langcode) {
    $enable_copy_button = \Drupal::config('highlightjs_input_filter.settings')->get('enable_copy_button');
    $result = new FilterProcessResult($text);
    if (stristr($text, '<code') === FALSE || stristr($text, 'language-') === FALSE) {
      return $result;
    }

    $pattern = "/language-[^\"]+/i";
    if (preg_match_all($pattern, $result, $matches)) {
      foreach ($matches as $match) {
        foreach ($match as $language_id) {
          $language = str_replace('language-', '', $language_id);
          // Create a uniquely keyed array of languages so the
          // BubbleableMetadata->mergeAttachments() method can properly
          // merge the drupalSettings when adding attachments.
          $languages[$language] = $language;
        }
      }
    }

    $result->addCacheableDependency($enable_copy_button);

    // Add needed JS.
    $result->addAttachments([
      'library' => [
        'highlightjs_input_filter/highlightjs',
        'highlightjs_input_filter/highlightjs_input_filter.styles',
      ],
      'drupalSettings' => [
        'enableCopyButton' => $enable_copy_button,
        'highlightJsLanguages' => $languages,
      ],
    ]);

    // Add copy button styles.
    if (isset($enable_copy_button) && $enable_copy_button === TRUE) {
      $result->addAttachments([
        'library' => [
          'highlightjs_input_filter/highlightjs-copy.styles',
        ],
      ]);
    }

    return $result;
  }

  /**
   * {@inheritdoc}
   */
  public function tips($long = FALSE) {
    return $this->t('Add assets for code tags');
  }

}
